-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

-- Copies configtable to saved setting
function BeneCast.SnapTo:ApplyConfigTable()

	if not wv.Setup then
		return;
	end

	if not wv.Setup.snapto then
		return;
	end
	
	local addon = BeneCast.SnapTo[wv.Setup.snapto];
	if addon then
		for pfid, pf in pairs(wv.Profiles) do
			pf.SnapTo = {};		
			for unit, config in pairs(addon.configtable) do
				pf.SnapTo[unit] = {
					point = config.point,
					relativepoint = config.relativepoint,
					x = config.x,
					y = config.y,
				};
				if config.orientation then
					pf.SnapTo[unit].orientation = config.orientation;
				else
					pf.SnapTo[unit].orientation = pf.orientation;
				end
			end
		end
	end

end

-- Generic function to attach panels based on saved setting
function BeneCast.SnapTo:SnapPanels()

	if not wv.Setup then
		return;
	end

	if not wv.Setup.snapto then
		return;
	end
	
	local addon = BeneCast.SnapTo[wv.Setup.snapto];
	if addon then
		local unitframe, profile, panelconf;
		for pfid, v in pairs(wv.Panels) do
			for i, unit in pairs(const.Targets) do
				if not v[unit] and not InCombatLockDown() then
					BeneCast.Panel:NewPanel(pfid,unit);
				end
				panel = v[unit];
				if panel then
					unitframe = addon:GetUnitFrame(unit);
					profile, panelconf = BeneCast:GetProfile(pfid, unit);
					if unitframe and panelconf then
						self:SnapPanel(unitframe,panel,panelconf);
					elseif not InCombatLockdown() then
						panel:Hide();
					end
				end
			end
		end
	end
	wv.AttachFrames = nil;
	
end

-- Function to attach a single panel to a unitframe
function BeneCast.SnapTo:SnapPanel(unitframe, panel, config)

	if unitframe == nil or panel == nil or config == nil then
		return;
	end
	
	-- Make sure we got nice rounded numbers
	config.x = math.floor(config.x + 0.5);
	config.y = math.floor(config.y + 0.5);
	
	if InCombatLockdown() then
		return;
	end
	
	UnregisterUnitWatch(panel);
	RegisterUnitWatch(panel);

	panel:SetParent(unitframe);
	panel:ClearAllPoints();
	local scaled_x = config.x / panel:GetEffectiveScale();
	local scaled_y = config.y / panel:GetEffectiveScale();
	panel:SetPoint(config.point, unitframe, config.relativepoint, scaled_x, scaled_y);
	panel:SetFrameLevel(unitframe:GetFrameLevel() + 1);
	-- Set up the correct level for the buttons on the panel
	local button = panel.next;
	while button do
		button:SetFrameLevel(panel:GetFrameLevel() + 1);
		button.Cooldown:SetFrameLevel(panel:GetFrameLevel() + 2);
	end
	wv.AttachedTargets[unit] = true;
	
	panel:OrientButtons();

end

-- Standard UI anchor positions
BeneCast.SnapTo.BeneCast = {
	name = 'Standard',
	configtable = {
		player       = { point = "TOPLEFT", frame = "PlayerFrame", relativepoint = "BOTTOM", x = -10, y = 30 },
		party1       = { point = "TOPLEFT", frame = "PartyMemberFrame1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "PartyMemberFrame2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "PartyMemberFrame3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "PartyMemberFrame4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "PartyMemberFrame1PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "PartyMemberFrame2PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "PartyMemberFrame3PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "PartyMemberFrame4PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "TargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		targettarget = { point = "TOPLEFT", frame = "TargetofTargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		focus        = { point = "TOPLEFT", frame = "PlayerFrame", relativepoint = "TOPRIGHT", x = 200, y = -20 },
	},
};

function BeneCast.SnapTo.BeneCast:GetUnitFrame(unit)
	if self.configtable[unit] then
		return getglobal(self.configtable[unit].frame);
	else
		return nil;
	end		
end

-- Perl anchor positions
BeneCast.SnapTo.Perl = {
	name = 'Perl',
	configtable = {
		player       = { point = "TOPLEFT", frame = "Perl_Player_Frame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "Perl_party1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "Perl_party2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "Perl_party3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "Perl_party4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "Perl_Party_Pet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "Perl_Party_Pet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "Perl_Party_Pet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "Perl_Party_Pet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "Perl_Target_Frame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		targettarget = { point = "TOPLEFT", frame = "Perl_TargetTarget_Frame", relativepoint = "BOTTOMLEFT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,
};

-- Perl anchor positions
BeneCast.SnapTo.Perl_Target = {
	name = 'Perl Classic',
	configtable = {
		player       = { point = "BOTTOMLEFT", frame = "Perl_Player_Frame", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		party1       = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame1", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		party2       = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame2", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		party3       = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame3", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		party4       = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame4", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame1_StatsFrame_PetHealthBar", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		partypet2    = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame2_StatsFrame_PetHealthBar", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		partypet3    = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame3_StatsFrame_PetHealthBar", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		partypet4    = { point = "BOTTOMLEFT", frame = "Perl_Party_MemberFrame4_StatsFrame_PetHealthBar", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		target       = { point = "BOTTOMLEFT", frame = "Perl_Target_Frame", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		focus        = { point = "BOTTOMLEFT", frame = "Perl_Focus_Frame", relativepoint = "TOPRIGHT", x = -5, y = -25 },
		targettarget = { point = "TOPLEFT", frame = "Perl_TargetTarget_Frame", relativepoint = "BOTTOMLEFT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,
};

-- Minigroup anchor positions
BeneCast.SnapTo.MiniGroup = {
	name = 'MiniGroup',
	configtable = {
		player       = { point = "TOPLEFT", frame = "MGParty_Member0", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "MGParty_Member1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "MGParty_Member2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "MGParty_Member3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "MGParty_Member4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "MGTarget_Frame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- Minigroup anchor positions
BeneCast.SnapTo.MiniGroup2 = {
	name = 'MiniGroup2',
	configtable = {
		player       = { point = "TOPLEFT", frame = "MGplayer", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "MGparty1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "MGparty2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "MGparty3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "MGparty4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "TOPLEFT", frame = "MGpet", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "MGpartypet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "MGpartypet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "MGpartypet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "MGpartypet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "MGtarget", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- Nurfed anchor positions
BeneCast.SnapTo.Nurfed = {
	name = 'Nurfed',
	configtable = {
		player       = { point = "TOPLEFT", frame = "Nurfed_player", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "Nurfed_party1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "Nurfed_party2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "Nurfed_party3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "Nurfed_party4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "Nurfed_partypet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "Nurfed_partypet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "Nurfed_partypet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "Nurfed_partypet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "Nurfed_target", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		targettarget = { point = "TOPLEFT", frame = "Nurfed_targettargettarget", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		focus        = { point = "TOPLEFT", frame = "Nurfed_focus", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- Noctambul anchor positions
BeneCast.SnapTo.NUF = {
	name = 'Noctambul',
	configtable = {
		player       = { point = "TOPLEFT", frame = "NUFPlayerFrame", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		party1       = { point = "TOPLEFT", frame = "NUFPartyMemberFrame1", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		party2       = { point = "TOPLEFT", frame = "NUFPartyMemberFrame2", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		party3       = { point = "TOPLEFT", frame = "NUFPartyMemberFrame3", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		party4       = { point = "TOPLEFT", frame = "NUFPartyMemberFrame4", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		pet          = { point = "TOPLEFT", frame = "NUFPetFrame", relativepoint = "BOTTOMLEFT", x = 0, y = 0, orientation = "right" },
		partypet1    = { point = "TOPLEFT", frame = "NUFPartyPet1", relativepoint = "BOTTOMLEFT", x = 0, y = 0, orientation = "right" },
		partypet2    = { point = "TOPLEFT", frame = "NUFPartyPet2", relativepoint = "BOTTOMLEFT", x = 0, y = 0, orientation = "right" },
		partypet3    = { point = "TOPLEFT", frame = "NUFPartyPet3", relativepoint = "BOTTOMLEFT", x = 0, y = 0, orientation = "right" },
		partypet4    = { point = "TOPLEFT", frame = "NUFPartyPet4", relativepoint = "BOTTOMLEFT", x = 0, y = 0, orientation = "right" },
		target       = { point = "TOPLEFT", frame = "NUFTargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
		targettarget = { point = "TOPLEFT", frame = "Dag_TargetTargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0, orientation = "down" },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- SAE_PartyFrame 
BeneCast.SnapTo.SAE_PartyFrame = {
	name = 'SAE_PartyFrame', 
	configtable = {
		player       = { point = "TOPLEFT", frame = "PlayerFrame", relativepoint = "BOTTOM", x = -10, y = 30 }, 
		party1       = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame1", relativepoint = "TOPRIGHT", x = -5, y = -5 }, 
		party2       = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame2", relativepoint = "TOPRIGHT", x = -5, y = -5 }, 
		party3       = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame3", relativepoint = "TOPRIGHT", x = -5, y = -5 }, 
		party4       = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame4", relativepoint = "TOPRIGHT", x = -5, y = -5 }, 
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 }, 
		partypet1    = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame1PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 2 }, 
		partypet2    = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame2PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 2 }, 
		partypet3    = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame3PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 2 }, 
		partypet4    = { point = "TOPLEFT", frame = "SAE_PartyMemberFrame4PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 2 }, 
		target       = { point = "LEFT", frame = "TargetFrame", relativepoint = "RIGHT", x = -20, y = 7 }, 
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- Watchdog anchor positions
BeneCast.SnapTo.WatchDog = {
	name = 'WatchDog',
	configtable = {
		player       = { point = "TOPLEFT", frame = "WatchDogFrame_player", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "WatchDogFrame_party1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "WatchDogFrame_party2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "WatchDogFrame_party3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "WatchDogFrame_party4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "WatchDogFrame_partypet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "WatchDogFrame_partypet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "WatchDogFrame_partypet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "WatchDogFrame_partypet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "WatchDogFrame_target", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- Discord anchor positions
BeneCast.SnapTo.DiscordUnitFrames = {
	name = 'Discord',
	configtable = {
		player       = { point = "TOPLEFT", frame = "DUF_PlayerFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "DUF_PartyFrame1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "DUF_PartyFrame2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "DUF_PartyFrame3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "DUF_PartyFrame4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "TOPLEFT", frame = "DUF_PetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "DUF_PartyPetFrame1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "DUF_PartyPetFrame2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "DUF_PartyPetFrame3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "DUF_PartyPetFrame4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "DUF_TargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		targettarget = { point = "TOPLEFT", frame = "DUF_TargetOfTargetFrame", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

--Sage anchor positions
BeneCast.SnapTo.Sage = {
	name = 'Sage',
	configtable = {
		player       = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = 0, y = -30 },
		party1       = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		party2       = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		party3       = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		party4       = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		pet          = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -110, y = -14 },
		partypet1    = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		partypet2    = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		partypet3    = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		partypet4    = { point = "TOPLEFT", relativepoint = "TOPRIGHT", x = -120, y = -14 },
		target       = { point = "TOPLEFT", relativepoint = "BOTTOMLEFT", x = 0, y = -20 },
		targettarget = { point = "TOPLEFT", relativepoint = "BOTTOMLEFT", x = 0, y = 0 },
		focus        = { point = "TOPLEFT", relativepoint = "BOTTOMLEFT", x = 0, y = 0 },
	},
};

function BeneCast.SnapTo.Sage:GetUnitFrame(unit)
	if self.configtable[unit] and SageFrame then
		for k, v in SageFrame.GetAll() do
    	if k == unit then
      	return v;
      end
    end
	end
end

-- ag_UnitFrames anchor positions
BeneCast.SnapTo.ag_UnitFrames = {
	name = 'ag_UnitFrames',
	configtable = {
		player       = { point = "TOPLEFT", frame = "aUFplayer", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "aUFpartyUnitButton1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "aUFpartyUnitButton2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "aUFpartyUnitButton3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "aUFpartyUnitButton4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "aUFpartypet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "aUFpartypet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "aUFpartypet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "aUFpartypet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "aUFtarget", relativepoint = "TOPRIGHT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};

-- XPerl anchor positions
BeneCast.SnapTo.XPerl  = {
	name = 'XPerl',
	configtable = {
		player       = { point = "TOPLEFT", frame = "XPerl_Player", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party1       = { point = "TOPLEFT", frame = "XPerl_party1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party2       = { point = "TOPLEFT", frame = "XPerl_party2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party3       = { point = "TOPLEFT", frame = "XPerl_party3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		party4       = { point = "TOPLEFT", frame = "XPerl_party4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		pet          = { point = "", frame = "", relativepoint = "", x = 0, y = 0 },
		partypet1    = { point = "TOPLEFT", frame = "XPerl_partypet1", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet2    = { point = "TOPLEFT", frame = "XPerl_partypet2", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet3    = { point = "TOPLEFT", frame = "XPerl_partypet3", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		partypet4    = { point = "TOPLEFT", frame = "XPerl_partypet4", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		target       = { point = "TOPLEFT", frame = "XPerl_Target", relativepoint = "TOPRIGHT", x = 0, y = 0 },
		targettarget = { point = "TOPLEFT", frame = "XPerl_TargetTarget", relativepoint = "BOTTOMLEFT", x = 0, y = 0 },
	},
	GetUnitFrame = BeneCast.SnapTo.BeneCast.GetUnitFrame,	
};
