-- Handy shortcuts
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;

-- Array to assign properties to spells and buffs that we can't deduce ourselves
-- Uses spellids for language-independance (Spellids acquired from thottbot and wowhead)
BeneCast.Spells = {
	-- All
	{ id = 28880, type = 'hot', target = 'all', flash = true }, -- Gift of the Naaru
	{ id = 4511,  type = 'buff_effect', uninteractible = true }, -- Phase Shift
	-- Druid
	{ id = 774,   type = 'hot', target = 'all', stance = { none = true, tree = true }, enables = { 5375 }, flash = true }, -- Rejuvenation                
	{ id = 467,   type = 'buff', target = 'all', stance = { none = true, moonkin = true } }, -- Thorns                   
	{ id = 5185,  type = 'heal', target = 'all', stance = { none = true, tree = true } }, -- Healing Touch               
	{ id = 5487,  type = 'stance', code = 'bear', group = 'bearform' }, -- Bear Form                             
	{ id = 1126,  type = 'buff', target = 'all', stance = { none = true, tree = true } }, -- Mark of the Wild            
	{ id = 16689, type = 'buff', target = 'self', stance = { none = true, moonkin = true } }, -- Nature's Grasp           
	{ id = 8936,  type = 'hot', target = 'all', stance = { none = true, tree = true }, enables = { 5375 }, flash = true }, -- Regrowth                    
	{ id = 50769, type = 'rez', target = 'all', stance = { none = true, tree = true } }, -- Revive                      
	{ id = 8946,  type = 'cure', target = 'all', cures = { 'poison' }, stance = { none = true, tree = true }, group = 'curepoison' }, -- Cure Poison   
	{ id = 1066,  type = 'stance', code = 'seal' }, -- Aquatic Form                                        
	{ id = 768,   type = 'stance', code = 'cat' }, -- Cat Form                                             
	{ id = 20484, type = 'brez', target = 'all', stance = { none = true, tree = true } }, -- Rebirth                     
	{ id = 2782,  type = 'cure', target = 'all', cures = { 'curse' }, stance = { none = true, moonkin = true } }, -- Remove Curse             
	{ id = 2893,  type = 'cure', target = 'all', cures = { 'poison' }, stance = { none = true, tree = true }, group = 'curepoison', flash = true }, -- Abolish Poison
	{ id = 740,   type = 'heal', target = 'self', stance = { none = true, tree = true } }, -- Tranquility                 
	{ id = 783,   type = 'stance', code = 'cheetah' }, -- Travel Form                                      
	{ id = 22842, type = 'buff', target = 'self', stance = { bear = true, direbear = true }, flash = true }, -- Frenzied Regeneration   
	{ id = 9634,  type = 'stance', code = 'direbear', group = 'bearform' }, -- Dire Bear Form                    
	{ id = 29166, type = 'buff', target = 'all', stance = { none = true, tree = true } }, -- Innervate                   
	{ id = 24858, type = 'stance', code = 'moonkin' }, -- Moonkin Form                                     
	{ id = 22812, type = 'buff', target = 'self', stance = { none = true, tree = true, moonkin = true }, flash = true }, -- Barkskin
	{ id = 21849, type = 'buff', target = 'group', stance = { none = true, tree = true }, overrides = { 1126 } }, -- Gift of the Wild
	{ id = 33891, type = 'stance', code = 'tree' }, -- Tree of Life Form
	{ id = 33763, type = 'hot', target = 'all', stance = { none = true, tree = true }, flash = true }, -- Lifebloom
	{ id = 33943, type = 'stance', code = 'flight', group = 'flightform' }, -- Flight Form
	{ id = 40120, type = 'stance', code = 'swiftflight', group = 'flightform' }, -- Swift Flight Form
	{ id = 50464, type = 'heal', target = 'all', stance = { none = true, tree = true } }, -- Nourish
	{ id = 5375,  type = 'consume', target = 'all', stance = { none = true, tree = true } }, -- Swiftmend
	{ id = 48438, type = 'heal', target = 'group', stance = { none = true, tree = true }, flash = true }, -- Wild Growth
	{ id = 17116, type = 'buff', target = 'self', stance = { none = true, tree = true } }, -- Nature's Swiftness
	-- Hunter
	{ id = 13161, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Beast
	{ id = 13163, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Monkey
	{ id = 13165, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Hawk
	{ id = 136,   type = 'heal', target = 'pet' }, -- Mend Pet
	{ id = 5116,  type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Cheetah 
	{ id = 34074, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Viper
	{ id = 3045,  type = 'buff', target = 'self' }, -- Rapid Fire
	{ id = 5384,  type = 'buff', target = 'self' }, -- Feign Death
	{ id = 13159, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Pack
	{ id = 19574, type = 'buff', target = 'pet' }, -- Bestial Wrath
	{ id = 20043, type = 'buff', target = 'self', sorting = 2 }, -- Aspect of the Wild
	{ id = 34026, type = 'buff', target = 'pet', }, -- Kill Command
	{ id = 34477, type = 'buff', target = 'grouponly', }, -- Misdirection
	{ id = 53271, type = 'buff', target = 'all' }, -- Master's Call
	{ id = 19577, type = 'buff', target = 'pet' }, -- Intimidation
	{ id = 23989, type = 'buff', target = 'self' }, -- Readiness
	{ id = 19506, type = 'buff', target = 'self' }, -- Trueshot Aura
	{ id = 31567, type = 'buff', target = 'self' }, -- Deterrence
	-- Mage
	{ id = 1459,  type = 'buff', target = 'all', group = 'arcaneintellect' }, -- Arcane Intellect
	{ id = 168,   type = 'buff', target = 'self', group = 'frost armor', sorting = 4 }, -- Frost Armor
	{ id = 604,   type = 'buff', target = 'group', sorting = 2 }, -- Dampen Magic
	{ id = 130,   type = 'buff', target = 'self' }, -- Slow Fall
	{ id = 1008,  type = 'buff', target = 'group', sorting  = 2 }, -- Amplify Magic
	{ id = 475,   type = 'cure', target = 'all', cures = { 'curse' } }, -- Remove Curse
	{ id = 12051, type = 'buff', target = 'self' }, -- Evocation
	{ id = 543,   type = 'buff', target = 'self', sorting = 3 }, -- Fire Ward
	{ id = 1463,  type = 'buff', target = 'self', sorting = 5 }, -- Mana Shield
	{ id = 6143,  type = 'buff', target = 'self', sorting = 3 }, -- Frost Ward
	{ id = 7302,  type = 'buff', target = 'self', group = 'frostarmor', sorting = 4 }, -- Ice Armor
	{ id = 45438, type = 'buff', target = 'self' }, -- Ice Block
	{ id = 6117,  type = 'buff', target = 'self', sorting = 4 }, -- Mage Armor
	{ id = 23028, type = 'buff', target = 'group', group = 'arcanebrilliance', overrides = { 1459 } }, -- Arcane Brilliance
	{ id = 30482, type = 'buff', target = 'self', sorting = 4 }, -- Molten Armor
	{ id = 66,    type = 'buff', target = 'self' }, -- Invisibility
	{ id = 61024, type = 'buff', target = 'all', group = 'arcaneintellect' }, -- Dalaran Intellect
	{ id = 61316, type = 'buff', target = 'group', group = 'arcanebrilliance', overrides = { 1459, 61024 } }, -- Dalaran Brilliance
	{ id = 54648, type = 'buff', target = 'self' }, -- Focus Magic
	{ id = 11426, type = 'buff', target = 'self', sorting = 5 }, -- Ice Barrier
	-- Paladin
	{ id = 19740, type = 'buff', target = 'all', sorting = 2 }, -- Blessing of Might
	{ id = 498,   type = 'buff', target = 'self', group = 'divineprotection' }, -- Divine Protection
	{ id = 635,   type = 'heal', target = 'all' }, -- Holy Light
	{ id = 1152,  type = 'cure', target = 'all', cures = { 'disease', 'poison' }, group = 'purify' }, -- Purify
	{ id = 1022,  type = 'buff', target = 'group', sorting = 4 }, -- Hand of Protection
	{ id = 25771, type = 'debuff_effect', overrides = { 498, 1022, 642 } }, -- Forbearance
	{ id = 633,   type = 'heal', target = 'all' }, -- Lay on Hands
	{ id = 7328,  type = 'rez', target = 'all' }, -- Redemption
	{ id = 19742, type = 'buff', target = 'all', sorting = 2 }, -- Blessing of Wisdom
	{ id = 31789, type = 'buff', target = 'allonly' }, -- Righteous Defense
	{ id = 25780, type = 'buff', target = 'self' }, -- Righteous Fury
	{ id = 1044,  type = 'buff', target = 'all', sorting = 4 }, -- Hand of Freedom
	{ id = 19750, type = 'heal', target = 'all' }, -- Flash of Light
	{ id = 21084, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Righteousness
	{ id = 20164, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Justice
	{ id = 19752, type = 'buff', target = 'grouponly' }, -- Divine Intervention
	{ id = 20165, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Light
	{ id = 642,   type = 'buff', target = 'self', group = 'divineprotection' }, -- Divine Shield	
	{ id = 20166, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Wisdom
	{ id = 4987,  type = 'cure', target = 'all', cures = { 'disease', 'poison', 'magic' }, group = 'purify' }, -- Cleanse
	{ id = 6940,  type = 'buff', target = 'grouponly', sorting = 4 }, -- Hand of Sacrifice
	{ id = 25782, type = 'buff', target = 'group', overrides = { 19740 }, sorting = 3 }, -- Greater Blessing of Might
	{ id = 25894, type = 'buff', target = 'group', overrides = { 19742 }, sorting = 3 }, -- Greater Blessing of Wisdom
	{ id = 20217, type = 'buff', target = 'all', sorting = 2 }, -- Blessing of Kings
	{ id = 25898, type = 'buff', target = 'group', overrides = { 20217 }, sorting = 3 }, -- Greater Blessing of Kings
	{ id = 20911, type = 'buff', target = 'all', sorting = 2 }, -- Blessing of Sanctuary
	{ id = 25899, type = 'buff', target = 'group', overrides = { 20911 }, sorting = 3 }, -- Greater Blessing of Sanctuary
	{ id = 53651, type = 'buff', target = 'all' }, -- Light's Beacon
	{ id = 31892, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Blood
	{ id = 31801, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Vengeance
	{ id = 53736, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Corruption
	{ id = 53720, type = 'buff', target = 'self', sorting = 5 }, -- Seal of the Martyr
	{ id = 31884, type = 'buff', target = 'self' }, -- Avenging Wrath
	{ id = 54428, type = 'buff', target = 'self' }, -- Divine Plea
	{ id = 53601, type = 'buff', target = 'all' }, -- Sacred Shield
	{ id = 20216, type = 'buff', target = 'self' }, -- Divine Favor
	{ id = 20473, type = 'heal', target = 'all' }, -- Holy Shock
	{ id = 20925, type = 'buff', target = 'self' }, -- Holy Shield
	{ id = 20375, type = 'buff', target = 'self', sorting = 5 }, -- Seal of Command	
	-- Priest
	{ id = 59544, type = 'hot',  target = 'all', stance = { none = true }, flash = true }, -- Gift of the Naaru
	{ id = 552,   type = 'cure', target = 'all', cures = { 'disease' }, stance = { none = true }, group = 'curedisease', flash = true }, -- Abolish Disease
	{ id = 32546, type = 'heal', target = 'allonly' }, -- Binding Heal
	{ id = 34861, type = 'heal', target = 'group', stance = { none = true } }, -- Circle of Healing
	{ id = 528,   type = 'cure', target = 'all', cures = { 'disease' }, stance = { none = true }, group = 'curedisease' }, -- Cure Disease
	{ id = 19236, type = 'heal', target = 'self', stance = { none = true } }, -- Desperate Prayer
	{ id = 527,   type = 'cure', target = 'all', cures = { 'magic' }, stance = 'all' }, -- Dispel Magic
	{ id = 47951, type = 'heal', target = 'self', stance = { none = true } }, -- Divine Hymn
	{ id = 14752, type = 'buff', target = 'all', stance = 'all' }, -- Divine Spirit
	{ id = 586,   type = 'buff', target = 'self', stance = 'all' }, -- Fade
	{ id = 6346,  type = 'buff', target = 'all', stance = 'all' }, -- Fear Ward
	{ id = 2061,  type = 'heal', target = 'all', stance = { none = true } }, -- Flash Heal
	{ id = 2050,  type = 'heal', target = 'all', group = 'lesserheal', stance = { none = true } }, -- Lesser Heal
	{ id = 2054,  type = 'heal', target = 'all', group = 'lesserheal', stance = { none = true } }, -- Heal
	{ id = 2060,  type = 'heal', target = 'all', group = 'lesserheal', stance = { none = true } }, -- Greater Heal
	{ id = 47788, type = 'buff', target = 'all', stance = { none = true } }, -- Guardian Spirit
	{ id = 15237, type = 'heal', target = 'self', stance = { none = true } }, -- Holy Nova
	{ id = 60931, type = 'buff', target = 'self', stance = { none = true } }, -- Hymn of Hope
	{ id = 588,   type = 'buff', target = 'self', stance = 'all' }, -- Inner Fire
	{ id = 1706,  type = 'buff', target = 'self', stance = 'all' }, -- Levitate
	{ id = 47666, type = 'heal', target = 'all', stance = 'all' }, -- Penance
	{ id = 1243,  type = 'buff', target = 'all', stance = 'all' }, -- Power Word: Fortitude
	{ id = 17,    type = 'buff', target = 'group', stance = 'all' }, -- Power Word: Shield
	{ id = 6788,  type = 'debuff_effect', overrides = { 17 } }, -- Weakened Soul
	{ id = 21562, type = 'buff', target = 'group', stance = 'all', overrides = { 1243 } }, -- Prayer of Fortitude
	{ id = 596,   type = 'heal', target = 'self', stance = { none = true } }, -- Prayer of Healing
	{ id = 33076, type = 'heal', target = 'all', stance = { none = true } }, -- Prayer of Mending
	{ id = 976,   type = 'buff', target = 'all', stance = 'all' }, -- Shadow Protection
	{ id = 27683, type = 'buff', target = 'group', stance = 'all', overrides = { 976 } }, -- Prayer of Shadow Protection
	{ id = 27681, type = 'buff', target = 'group', stance = 'all', overrides = { 14752 } }, -- Prayer of Spirit
	{ id = 139,   type = 'hot', target = 'all', stance = { none = true }, flash = true }, -- Renew
	{ id = 2006,  type = 'rez', target = 'all', stance = { none = true } }, -- Resurrection
	{ id = 15473, type = 'stance', code = 'shadow' }, -- Shadowform
	{ id = 47585, type = 'buff', target = 'self', stance = 'all' }, -- Dispersion
	{ id = 14751, type = 'buff', target = 'self', stance = 'all' }, -- Inner Focus
	-- Rogue
	{ id = 1784,  type = 'stance', code = 'stealth' }, -- Stealth
	{ id = 5277,  type = 'buff', target = 'self', stance = 'all' }, -- Evasion
	{ id = 2983,  type = 'buff', target = 'self', stance = 'all' }, -- Sprint
	{ id = 1856,  type = 'buff', target = 'self', stance = 'all' }, -- Vanish
	{ id = 31224, type = 'cure', cures = { 'posion', 'magic', 'disease', 'curse' }, target = 'self', stance = { none = true } }, -- Cloak of Shadows
	{ id = 57934, type = 'buff', target = 'grouponly', stance = { none = true } }, -- Tricks of the Trade
	{ id = 13750, type = 'buff', target = 'self', stance = { none = true } }, -- Adrenaline Rush
	{ id = 13877, type = 'buff', target = 'self', stance = { none = true } }, -- Blade Flurry
	{ id = 14177, type = 'buff', target = 'self', stance = 'all' }, -- Cold Blood
	{ id = 51662, type = 'buff', target = 'self', stance = { none = true } }, -- Hunger for Blood
	{ id = 14183, type = 'buff', target = 'self', stance = { stealth = true } }, -- Premeditation
	{ id = 14185, type = 'buff', target = 'self', stance = { none = true } }, -- Preparation
	{ id = 51713, type = 'buff', target = 'self', stance = { none = true } }, -- Shadow Dance
		
};

BeneCast.Groups = {
	bearform = { 5487, 9634 }, -- Bear Form, Dire Bear Form
	curepoison = { 8946, 2893 }, -- Cure Poison, Abolish Poison
	flightform = { 33943, 40120 }, -- Flight Form, Swift Flight Form
	frostarmor = { 168, 7302 }, -- Frost Armor, Ice Armor
	arcaneintellect = { 1459, 61024 }, -- Arcane Intellect, Dalaran Intellect
	arcanebrilliance = { 23028, 61316 }, -- Arcane Brilliance, Dalaran Brilliance
	divineprotection = { 498, 642 }, -- Divine Protection, Divine Shield
	purify = { 1152, 4987 }, -- Purify, Cleanse
	curedisease = { 528, 552 }, -- Cure Disease, Abolish Disease
	lesserheal = { 2050, 2054, 2060 }, -- Lesser Heal, Heal, Greater Heal 
};

-- Function to translate the spellids into their proper names
function BeneCast:FillSpellInfo()
	wv.SpellInfo = hp._ClearTable(wv.SpellInfo, true);
	wv.GroupInfo = hp._ClearTable(wv.GroupInfo, true);
	
	-- Translate the groups to name-form first
	local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange;
	local groupcodetoname = {};
	for code, grp in pairs(self.Groups) do
		name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(grp[1]);
		wv.GroupInfo[name] = hp._CopyTable(grp);
		groupcodetoname[code] = name;
	end
	
	-- Then translate the spells themselves
	local spellinfo;
	local hasoverrides = {};
	local hasenables = {};
	for i, spell in pairs(self.Spells) do
		name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spell.id);
		if name then
			spell.name = name;
			spellinfo = hp._CopyTable(spell,true);
			spellinfo.rank = rank;
			spellinfo.icon = icon;
			spellinfo.minrange = minRange;
			spellinfo.maxrange = maxRange;
			if spell.group then
				spellinfo.group = groupcodetoname[spell.group];
			else
				spellinfo.group = name;
			end
			wv.SpellInfo[name] = spellinfo;
			if spellinfo.overrides then
				table.insert(hasoverrides,spellinfo);
			end
			if spellinfo.enables then
				table.insert(hasenables,spellinfo);
			end
		else
			self:Print('No name found for spellid ' .. spell.id);
		end
	end
	
	-- The overrides also need to be translated since the matching with buffs is done by name
	local dummy;
	for i, spellinfo in pairs(hasoverrides) do
		dummy = spellinfo.overrides;
		spellinfo.overrides = {};
		for j, id in pairs(dummy) do
			spell = self.Spells[id];
			if spell then
				table.insert(spellinfo.overrides,spell.name);
			end
		end
		dummy = nil;
	end
	-- Same for the enables
	for i, spellinfo in pairs(hasenables) do
		dummy = spellinfo.enables;
		spellinfo.enables = {};
		for j, id in pairs(dummy) do
			spell = self.Spells[id];
			if spell then
				table.insert(spellinfo.enables,spell.name);
			end
		end
		dummy = nil;
	end
end

-- Fill in the wv.Spells table by iterating over spells in spellbook
function BeneCast:LoadSpellData()
	self:Debug('Parsing Spell Data from spellbook');
	
	wv.Spells = hp._ClearTable(wv.Spells, true);
	wv.TempEnchantsInfo = hp._ClearTable(wv.TempEnchantsInfo, true);
	
	-- Iterator to move through the spellbook
	local i = 1;
	local spell_name, spell_rank, rankno;
	local spell, oldspell, spellinfo, rootid;
	local spellgroup;

	while true do
		
		-- Spell at position i in the player's spellbook
		spell_name, spell_rank = GetSpellName(i, SpellBookFrame.bookType);
		
		-- If there is no spell at i, break out of the loop
		if not spell_name then
			do break end
		end
		
		_, _, rankno = string.find(spell_rank,'%d+'); -- Find any string of numbers in the rank
		if rankno then
			rankno = tonumber(rankno);
		end
			
		spellinfo = wv.SpellInfo[spell_name];
		if spellinfo then
			spell = {
				spell = true,
				name = spell_name,
				rank = spell_rank,
				rankno = rankno,
				spellbookid = i,
				infoid = spellinfo.id,
				group = spellinfo.group,
				groupid = 0,
				type = spellinfo.type,
				target = spellinfo.target,
				stances = hp._CopyTable(spellinfo.stances,false),
				code = spellinfo.code,
				cures = hp._CopyTable(spellinfo.cures,false),
				overrides = hp._CopyTable(spellinfo.overrides,false),
				icon = spellinfo.icon,
				minrange = spellinfo.minrange,
				maxrange = spellinfo.maxrange,
				sorting = spellinfo.sorting,
				uninteractible = spellinfo.uninteractible,
				flash = spellinfo.flash,
				enchant = spellinfo.enchant,
			};
			if spell.sorting == nil then
				spell.sorting = 1;
			end
			if BeneCast.GroupInfo[spell.group] then
				for grpid, spellid in pairs(BeneCast.GroupInfo[spell.group]) do
					if spellid == spell.infoid then
						spell.groupid = grpid;
					end
				end
			end
			oldspell = wv.Spells[spell.group];
			if oldspell then	
				if (spell.groupid > oldspell.groupid) or (spell.groupid == oldspell.groupid and spell.rankno > oldspell.rankno) then
					wv.Spells[spell.group] = spell;
				end
			else
				wv.Spells[spell.group] = spell;
			end
						
		end

		i = i + 1;

	end
	
	-- Store enchants seperately since they have to be evaluated often
	for group, spell in pairs(wv.Spells) do
		if spell.enchant then
			wv.TempEnchantsInfo[spell.name] = group;
		end
	end
	
end